<?php declare(strict_types=1);

namespace Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Objects\ContentPartItem;
use Infomaker\Everyware\NewsML\Item;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use SimpleXMLIterator;

/**
 * ContentPartParser
 * @package Infomaker\Everyware\NewsML\Parsers
 */
class ContentPartParser implements ItemParser, TypeParser
{

    public const OBJECT_TYPE = 'x-im/content-part';

    public const DEFAULT_DATA = [
        'title' => '',
        'text' => [],
        'subject' => ''
    ];

    /**
     * @var LinkParser
     */
    protected $linkParser;

    /**
     * @var ItemParser[]
     */
    private $textParsers;

    public function __construct(ElementParser $parser = null)
    {
        $this->textParsers = [
            ElementParser::TAG => $parser ?? new ElementParser()
        ];

        $this->linkParser = new LinkParser([
            ImageLinkParser::LINK_TYPE => new ImageLinkParser()
        ]);
    }

    public function parse(SimpleXMLIterator $element): array
    {
        $item = new ContentPartItem($element);
        $links = $this->linkParser->parseLinks($element);
        $item->fill(array_replace(
            $this->getData($item),
            $this->extractLinkData($links)
        ));

        return [$item];
    }

    protected function getData(Item $item): array
    {
        $allowedTags = [
            'mark',
            'em',
            'strong'
        ];

        $data = $item->getFirstChild('data');

        if ( ! $data instanceof Item) {
            return static::DEFAULT_DATA;
        }

        $optionalData = array_filter([
            'title' => $data->getFirstChildAsXml('title', $allowedTags),
            'text' => $this->parseText($data, $allowedTags),
            'subject' => $data->getFirstChildAsXml('subject', $allowedTags),
        ]);

        return array_replace(static::DEFAULT_DATA, $optionalData);
    }

    protected function parseText(Item $dataItem, $allowedTags): array
    {
        $textItem = $dataItem->getFirstChild('text');

        if ( ! $textItem instanceof Item) {
            return [];
        }

        if ($textItem->get('format', '') !== 'idf') {
            $textItem->fill([
                'type' => 'body',
                'content' => $textItem->getFilteredContent($allowedTags)
            ]);

            return [$textItem];
        }

        $items = [[]];

        try {
            $textIterator = new SimpleXMLIterator($textItem->getContentAsXML());
        } catch (\Exception $e) {
            return [];
        }

        foreach ($textIterator as $tag => $el) {
            $parser = $this->textParsers[$tag] ?? null;
            $items[] = $parser instanceof ItemParser ? $parser->parse($el) : [new Item($el)];
        }

        return array_merge(...$items);
    }

    protected function extractLinkData(array $links): array
    {
        if (empty($links)) {
            return [];
        }

        $contentType = '';
        foreach ($links as $link) {
            if ($link instanceof Item) {
                $attributes = $link->getItemAttributes();
                if (($attributes['rel'] ?? '') === 'content-part') {
                    $contentType = str_replace('im://content-part/', '', $attributes['uri'] ?? '');
                }
            }
        }

        return [
            'links' => $links,
            'content_type' => $contentType
        ];
    }

    public function typeToParse(): string
    {
        return static::OBJECT_TYPE;
    }
}
