<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\ContentPartParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\ItemParser;
use Infomaker\Everyware\NewsML\Parsers\Interfaces\TypeParser;
use Infomaker\Everyware\Support\Format;
use Infomaker\Everyware\NewsML\Parsers\ElementParser;
use Spec\Infomaker\Everyware\NewsML\ParserSpec;

class ContentPartParserSpec extends ParserSpec
{

    function let()
    {
        $this->beConstructedWith(new ElementParser());
    }

    function it_is_initializable()
    {
        $this->shouldHaveType(ContentPartParser::class);
        $this->shouldImplement(ItemParser::class);
        $this->shouldImplement(TypeParser::class);
    }

    public function it_can_parse_an_element()
    {
        $xml = <<<XML
<object id="MTUwLDE4Miw1NCwxMjc" type="x-im/content-part" title="Plain-text title" uuid="778d7968-35e3-4347-b085-c4724ae8c885">
    <data>
        <title>Title with annotations</title>
        <subject>Lorem ipsum dolor sit amet, consectetur adipiscing elit</subject>
        <text format="idf">
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">Text element here</element>
        </text>
        <customField>2017-01-30T12:00:00+01:00</customField>
        <alignment>right</alignment>
    </data>
    <links>
        <link rel="image" type="x-im/image" uri="im://image/WXuZc-3d9_Iet5x5t3-LT0X4hWE.jpg" uuid="7d6450cd-27a6-589d-a84a-d6bb635cb542">
            <data>
                <width>1080</width>
                <height>1080</height>
            </data>
            <links>
                <link title="16:9" rel="crop" type="x-im/crop" uri="im://crop/0/0.21875/1/0.5625"/>
                <link title="1:1" rel="crop" type="x-im/crop" uri="im://crop/0/0/1/1"/>
                <link rel="focal-point" type="x-im/focal-point">
                    <data>
                        <x>123</x>
                        <y>456</y>
                    </data>
                </link>
            </links>
        </link>
        <link uri="im://content-part/fact" rel="content-part"/>
    </links>
</object>
XML;
        $this->validateXML($xml, [
            'type' => 'x-im/content-part',
            'title' => 'Title with annotations',
            'subject' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit',
            'content_type' => 'fact',
            'text' => [
                ['content' => 'Text element here']
            ],
            'links' => [
                [
                    'type' => 'x-im/image',
                    'uuid' => '7d6450cd-27a6-589d-a84a-d6bb635cb542',
                    'width' => '1080',
                    'height' => '1080',
                    'crop' => [
                        '16:9' => [
                            'x' => Format::formatNumberToImengineFloat(0),
                            'y' => Format::formatNumberToImengineFloat(0.21875),
                            'width' => Format::formatNumberToImengineFloat(1),
                            'height' => Format::formatNumberToImengineFloat(0.5625),
                        ],
                        '1:1' => [
                            'x' => Format::formatNumberToImengineFloat(0),
                            'y' => Format::formatNumberToImengineFloat(0),
                            'width' => Format::formatNumberToImengineFloat(1),
                            'height' => Format::formatNumberToImengineFloat(1),
                        ],
                    ],
                    'focal-point' => [
                        ['x' => '123', 'y' => '456']
                    ]
                ]
            ]
        ]);
    }

    public function it_can_return_type_to_be_parsed()
    {
        $this->typeToParse()
            ->shouldReturn('x-im/content-part');
    }

    public function it_can_get_content_type()
    {
        $xml = <<<XML
<object id="MTUwLDE4Miw1NCwxMjc" type="x-im/content-part" title="Plain-text title" uuid="778d7968-35e3-4347-b085-c4724ae8c885">
    <data>
        <title>Title with annotations</title>
        <subject>Lorem ipsum dolor sit amet, consectetur adipiscing elit</subject>
        <text format="idf">
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">Text element here</element>
        </text>
    </data>
    <links>
        <link uri="im://content-part/fact" rel="content-part"/>
    </links>
</object>
XML;
        $this->validateXML($xml, [
            'content_type' => 'fact'
        ]);
    }

    public function it_should_return_defaultData_if_data_is_none_existing()
    {
        $xml = <<<XML
<object id="MTUwLDE4Miw1NCwxMjc" type="x-im/content-part" title="Plain-text title" uuid="778d7968-35e3-4347-b085-c4724ae8c885">
    <data/>
    <links>
        <link uri="im://content-part/fact" rel="content-part"/>
    </links>
</object>
XML;
        $this->validateXML($xml, [
            'title' => '',
            'subject' => '',
            'text' => []
        ]);
    }

    public function it_should_parse_none_idf_text()
    {
        $xml = <<<XML
<object id="MTUwLDE4Miw1NCwxMjc" type="x-im/content-part" title="Plain-text title" uuid="778d7968-35e3-4347-b085-c4724ae8c885">
    <data>
        <title>Title with annotations</title>
        <subject>Lorem ipsum dolor sit amet, consectetur adipiscing elit</subject>
        <text>Paragraph text <strong>here</strong></text>
    </data>
    <links>
        <link uri="im://content-part/fact" rel="content-part"/>
    </links>
</object>
XML;
        $this->validateXML($xml, [
            'type' => 'x-im/content-part',
            'title' => 'Title with annotations',
            'subject' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit',
            'content_type' => 'fact',
            'text' => [
                ['type' => 'body', 'content' => 'Paragraph text <strong>here</strong>']
            ]
        ]);
    }

    public function it_should_parse_title_with_inline_tags()
    {
        $title = '<strong>Title</strong> <em>with</em> <mark>annotations</mark>';
        $xml = <<<XML
<object id="MTUwLDE4Miw1NCwxMjc" type="x-im/content-part" title="Plain-text title" uuid="778d7968-35e3-4347-b085-c4724ae8c885">
    <data>
        <title>$title</title>
        <subject>Lorem ipsum dolor sit amet, consectetur adipiscing elit</subject>
        <text format="idf">
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="headline">Headline <strong>here</strong></element>
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">First paragraph <strong>here</strong></element>
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">Second paragraph here</element>
        </text>
    </data>
    <links>
        <link uri="im://content-part/fact" rel="content-part"/>
    </links>
</object>
XML;
        $this->validateXML($xml, [
            'type' => 'x-im/content-part',
            'title' => $title,
            'subject' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit',
            'content_type' => 'fact',
            'text' => [
                ['type' => 'headline', 'content' => 'Headline <strong>here</strong>'],
                ['type' => 'body', 'content' => 'First paragraph <strong>here</strong>'],
                ['type' => 'body', 'content' => 'Second paragraph here'],
            ]
        ]);
    }



    public function it_should_parse_subject_with_inline_tags()
    {
        $subject = '<strong>Subject</strong> <em>with</em> <mark>annotations</mark>';
        $xml = <<<XML
<object id="MTUwLDE4Miw1NCwxMjc" type="x-im/content-part" title="Plain-text title" uuid="778d7968-35e3-4347-b085-c4724ae8c885">
    <data>
        <title>Lorem ipsum dolor sit amet, consectetur adipiscing elit</title>
        <subject>$subject</subject>
        <text format="idf">
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="headline">Headline <strong>here</strong></element>
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">First paragraph <strong>here</strong></element>
            <element id="paragraph-273ee570c1469bdf5badeea5f0524166" type="body">Second paragraph here</element>
        </text>
    </data>
    <links>
        <link uri="im://content-part/fact" rel="content-part"/>
    </links>
</object>
XML;
        $this->validateXML($xml, [
            'type' => 'x-im/content-part',
            'title' => 'Lorem ipsum dolor sit amet, consectetur adipiscing elit',
            'subject' => $subject,
            'content_type' => 'fact',
            'text' => [
                ['type' => 'headline', 'content' => 'Headline <strong>here</strong>'],
                ['type' => 'body', 'content' => 'First paragraph <strong>here</strong>'],
                ['type' => 'body', 'content' => 'Second paragraph here'],
            ]
        ]);
    }
}
