<?php

namespace Spec\Infomaker\Everyware\NewsML\Parsers;

use Infomaker\Everyware\NewsML\Parsers\TableParser;
use Infomaker\Everyware\NewsML\Item;
use PhpSpec\ObjectBehavior;
use Prophecy\Argument;
use Spec\Infomaker\Everyware\NewsML\ParserSpec;

class TableParserSpec extends ParserSpec
{
    function it_is_initializable()
    {
        $this->shouldHaveType(TableParser::class);
    }

    public function it_should_parse_an_object()
    {
        $head = <<<HEAD
<thead>
    <tr>
        <th id="table-cell-1" colspan="4">This is the (optional) head of the table</th>
    </tr>
</thead>
HEAD;
        $body = <<<BODY
<tbody>
    <tr>
        <td id="table-cell-2">Cell 1</td>
        <td id="table-cell-3">Cell 2</td>
        <td id="table-cell-4" rowspan="2">Cell 3 (with a rowspan of 2)</td>
    </tr>
    <tr>
        <td id="table-cell-5">Cell 1</td>
        <td id="table-cell-6">Cell 2</td>
        <td id="table-cell-7">Cell 3</td>
        <td id="table-cell-8">Cell 4</td>
    </tr>
</tbody>
BODY;
        $foot = <<<FOOT
<tfoot>
    <tr>
        <td id="table-cell-9" colspan="4">This is the (optional) foot of the table</td>
    </tr>
</tfoot>
FOOT;
        $caption = <<<CAPTION
<caption>This is the (optional) caption of the table</caption>
CAPTION;

        $xml = <<<XML
<object id="table-1" type="x-im/table">
    <data>
        $caption
        $head
        $body
        $foot
    </data>
</object>
XML;

        $this->validateXML($xml, [
            'type' => 'x-im/table',
            'id' => 'table-1',
            'caption' => $this->concatenateXML($caption),
            'head' => $this->concatenateXML($head),
            'body' => $this->concatenateXML($body),
            'foot' => $this->concatenateXML($foot),
        ]);
    }

    public function it_should_have_default_values_when_data_is_missing()
    {
        $xml = '<object type="x-im/table"><data/></object>';
        $this->validateXML($xml, [
            'type' => 'x-im/table',
            'caption' => '',
            'head' => '',
            'body' => '',
            'foot' => '',
        ]);

        $xml = '<object type="x-im/table"></object>';
        $this->validateXML($xml, [
            'type' => 'x-im/table',
            'caption' => '',
            'head' => '',
            'body' => '',
            'foot' => '',
        ]);
    }

    public function it_should_parse_a_caption_with_annotation()
    {
        $caption = <<<CAPTION
<caption>
    <strong>Str</strong> and <em>em</em>
</caption>
CAPTION;

        $xml = <<<XML
        <object type="x-im/table">
            <data>
                $caption
            </data>
        </object>
XML;

        $this->validateXML($xml, [
            'caption' => $this->concatenateXML($caption)
        ]);
    }

    public function it_should_parse_a_tbody_with_annotation()
    {
        $body = <<<BODY
<tbody>
    <tr>
        <td id="table-cell-1"><strong>Cell 1</strong></td>
        <td id="table-cell-2"><em>Cell 2</em></td>
        <td id="table-cell-3">Cell 3 (with a rowspan of 2)</td>
        <td id="table-cell-4">Cell 4</td>
    </tr>
</tbody>
BODY;
        $xml = <<<XML
        <object type="x-im/table">
            <data>
                $body
            </data>
        </object>
XML;

        $this->validateXML($xml, [
            'body' => $this->concatenateXML($body)
        ]);
    }

    public function it_should_parse_a_thead_with_annotation()
    {
        $head = <<<HEAD
<thead>
    <tr>
        <th id="table-cell-1"><strong>Cell 1</strong></th>
        <th id="table-cell-2"><em>Cell 2</em></th>
        <th id="table-cell-3">Cell 3 (with a rowspan of 2)</th>
        <th id="table-cell-4">Cell 4</th>
    </tr>
</thead>
HEAD;
        $xml = <<<XML
        <object type="x-im/table">
            <data>
                $head
            </data>
        </object>
XML;

        $this->validateXML($xml, [
            'head' => $this->concatenateXML($head)
        ]);
    }

    public function it_should_parse_a_tfoot_with_annotation()
    {
        $foot = <<<FOOT
<tfoot>
    <tr>
        <td id="table-cell-1"><strong>Cell 1</strong></td>
        <td id="table-cell-2"><em>Cell 2</em></td>
        <td id="table-cell-3">Cell 3 (with a rowspan of 2)</td>
        <td id="table-cell-4">Cell 4</td>
    </tr>
</tfoot>
FOOT;
        $xml = <<<XML
        <object type="x-im/table">
            <data>
                $foot
            </data>
        </object>
XML;

        $this->validateXML($xml, [
            'foot' => $this->concatenateXML($foot),
        ]);
    }

    public function it_should_return_its_type()
    {
        $this->typeToParse()->shouldReturn('x-im/table');
    }
}
